/*  SATI-3SXのデータからCT2等を計算するプログラム
    制作:2012/02/20 沖田博文 (東北大・天文・D2) 
    コンパイルは以下のコマンドで実行 
    $ gcc -lm -o CT2 ct2.c */

#include <stdio.h>
#include <math.h>

#define N 600	/* 観測回数、20Hz x 30sec = 600 (暫定値) */
#define X 1.0   /* メジアン温度との差のスレッシュホールド、1度(暫定値) */

int main(int argc, char *argv[]){

    FILE *fi;
    int i, j, m=0, n=0;
    double t[N], u[N], v[N], w[N], T[N], c[N], V[N], Tm[N], Vm[N];
    double Tmed=0.0, Vmed=0.0, hoge=0.0, CT=0.0;
        
    /* ファイル読み込み */
    fi=fopen(argv[1], "r");
    for(i=0; i<N; i++){
        fscanf(fi, "%lf%lf%lf%lf%lf%lf\n",&t[i],&u[i],&v[i],&w[i],&T[i],&c[i]);
        V[i] = sqrt(u[i]*u[i] + v[i]*v[i] + w[i]*w[i]);
        Tm[i]=T[i];
        Vm[i]=V[i];
//        printf("%.8lf\t%.2lf\n", V[i], T[i]);
    }
    fclose(fi);

    //温度のMedian値Tmを計算
    for(j=1; j<N; j++){
        for(i=0; i<N-j; i++){
            if(Tm[i] < Tm[i+1]){
                hoge = Tm[i];
                Tm[i] = Tm[i+1];
                Tm[i+1] = hoge;
            }
            if(Vm[i] < Vm[i+1]){
                hoge = Vm[i];
                Vm[i] = Vm[i+1];
                Vm[i+1] = hoge;
            }
        }
    }
//    for(i=0; i<N; i++){
//        printf("%.8lf\t%.2lf\n", V[i], Tm[i]);
//    }
    if(N%2==1){
        Tmed = Tm[(N-1)/2];
        Vmed = Vm[(N-1)/2];
    }
    else{
        Tmed = (Tm[(N-1)/2]+Tm[(N-1)/2+1])/2.0;
        Vmed = (Vm[(N-1)/2]+Vm[(N-1)/2+1])/2.0;
    }
//    printf("%.2lf\n", Tmed);

    //CT2の値を計算
    for(i=0;i<N;i++){
        if( pow((T[i]-Tmed),2.0) > pow(X,2.0) ){
            m=0;
        }
        else{
            if(m==0){
                /* 1回目 */
                m=1;
            }
            else{
                /* 2回目以降 */
                CT = CT + pow((T[i]-T[i-1]),2.0) / pow(((t[i]-t[i-1])*(V[i-1]+V[i])/2.0), 2/3.0);
               n=n+1;
            }
        }
    }
    CT=CT/n;
    printf("%lf\t%0.2lf\t%0.2lf\t%d\t%.10lf\n",t[0],Vmed,Tmed,n,CT);

    return 0;
}
