/*  Applied Technologies, Inc. Sonic Anemometer/Thermometer SATI-3SXからデータを取得するプログラム
    制作:2012/01/29 沖田博文 (東北大・天文・D2) 
    コンパイルは以下のコマンドで実行 
    $ gcc -lm -o sonic rs232c_sonic.c */

#include <stdio.h>	/* 入出力標準ライブラリ */
#include <termios.h>	/* 非同期通信ポートを制御する為の汎用ターミナルインターフェース */
#include <fcntl.h>	/* ファイル操作(低水準入出力関数) */
#include <unistd.h>	/* standard symbolic constants and types */
#include <sys/time.h>	/* for gettimeofday() */

int main(int argc, char *argv[]){
//int main(void){

    struct termios tio;
    int fd;
    FILE *fpp;
    char callback[256];

    double now=0.0, start=0.0;
    struct timeval t, s;

    /* ファイルオープン */
    fd=open("/dev/ttyUSB0",O_RDWR | O_NOCTTY | O_NDELAY);
    fpp=fopen("/dev/ttyUSB0","r");

    if(fd<0){
        printf("Error: Cannot open the Serial Device\n");
        return -1;
    }

    /* シリアルポートの設定 */
    tio.c_cflag = B9600 | CS7 | PARENB | CREAD | CLOCAL;	/* control modes */
    /*Baud rate 9600, Parity even, Data bits 1, Stop bit 1, flow control none
      詳しくはターミナルで man termiosして調べる*/
    tio.c_iflag = 0;					/* input modes   */
    tio.c_oflag = 0;					/* output modes  */
    tio.c_lflag = 0;			/* local modes   */
    tio.c_cc[VMIN] = 0;		/* Minimum number of characters for noncanonical read, 0にする */
    tio.c_cc[VTIME] = 1;		/* Timeout in deciseconds for noncanonical read, 1以上にする */

    tcsetattr(fd, TCSANOW, &tio);	/* 端末に関連したパラメータの設定(ハードウェアの設定に必要), ただちに変更を有効 */

    /* 受信 */
    gettimeofday(&s,NULL);
    start=s.tv_sec+s.tv_usec/1000000.0;
    while(1){
        fgets(callback,sizeof(callback),fpp);	/* RS232Cからデータの読み込み */
        gettimeofday(&t,NULL);			/* 現在時刻の取得 */
        now=t.tv_sec+t.tv_usec/1000000.0;
        printf("%lf  %s",now,callback);

        /* 測定開始からの時間を計算し、もしargv[1]sec以上なら測定を終了 */
        if((now-start) > atoi(argv[1])) break;
    }
    /* ファイルクローズ */
    close(fd);
    fclose(fpp);

    return 0;
}
